package java.net;

import sun.net.ResourceManager;
import java.io.IOException;
import java.io.FileDescriptor;

class TwoStacksPlainDatagramSocketImpl extends AbstractPlainDatagramSocketImpl
{
    private FileDescriptor fd1;
    private InetAddress anyLocalBoundAddr;
    private int fduse;
    private int lastfd;
    private final boolean exclusiveBind;
    private boolean reuseAddressEmulated;
    private boolean isReuseAddress;
    
    TwoStacksPlainDatagramSocketImpl(boolean exclusiveBind) {
        this.anyLocalBoundAddr = null;
        this.fduse = -1;
        this.lastfd = -1;
        this.exclusiveBind = exclusiveBind;
    }
    
    @Override
    protected synchronized void create() throws SocketException {
        this.fd1 = new FileDescriptor();
        try {
            super.create();
        }
        catch (SocketException ex) {
            this.fd1 = null;
            throw ex;
        }
    }
    
    @Override
    protected synchronized void bind(int n, InetAddress anyLocalBoundAddr) throws SocketException {
        super.bind(n, anyLocalBoundAddr);
        if (anyLocalBoundAddr.isAnyLocalAddress()) {
            this.anyLocalBoundAddr = anyLocalBoundAddr;
        }
    }
    
    @Override
    protected synchronized void bind0(int n, InetAddress inetAddress) throws SocketException {
        this.bind0(n, inetAddress, this.exclusiveBind);
    }
    
    @Override
    protected synchronized void receive(DatagramPacket datagramPacket) throws IOException {
        try {
            this.receive0(datagramPacket);
        }
        finally {
            this.fduse = -1;
        }
    }
    
    @Override
    public Object getOption(int n) throws SocketException {
        if (this.isClosed()) {
            throw new SocketException("Socket Closed");
        }
        if (n == 15) {
            if (this.fd != null && this.fd1 != null && !this.connected) {
                return this.anyLocalBoundAddr;
            }
            return this.socketLocalAddress((this.connectedAddress == null) ? -1 : this.connectedAddress.holder().getFamily());
        }
        else {
            if (n == 4 && this.reuseAddressEmulated) {
                return Boolean.valueOf(this.isReuseAddress);
            }
            return super.getOption(n);
        }
    }
    
    @Override
    protected void socketSetOption(int n, Object o) throws SocketException {
        if (n == 4 && this.exclusiveBind && this.localPort != 0) {
            this.reuseAddressEmulated = true;
            this.isReuseAddress = (boolean)o;
        }
        else {
            this.socketNativeSetOption(n, o);
        }
    }
    
    @Override
    protected boolean isClosed() {
        return this.fd == null && this.fd1 == null;
    }
    
    @Override
    protected void close() {
        if (this.fd != null || this.fd1 != null) {
            this.datagramSocketClose();
            ResourceManager.afterUdpClose();
            this.fd = null;
            this.fd1 = null;
        }
    }
    
    protected synchronized native void bind0(int p0, InetAddress p1, boolean p2) throws SocketException;
    
    @Override
    protected native void send(DatagramPacket p0) throws IOException;
    
    @Override
    protected synchronized native int peek(InetAddress p0) throws IOException;
    
    @Override
    protected synchronized native int peekData(DatagramPacket p0) throws IOException;
    
    @Override
    protected synchronized native void receive0(DatagramPacket p0) throws IOException;
    
    @Override
    protected native void setTimeToLive(int p0) throws IOException;
    
    @Override
    protected native int getTimeToLive() throws IOException;
    
    @Deprecated
    @Override
    protected native void setTTL(byte p0) throws IOException;
    
    @Deprecated
    @Override
    protected native byte getTTL() throws IOException;
    
    @Override
    protected native void join(InetAddress p0, NetworkInterface p1) throws IOException;
    
    @Override
    protected native void leave(InetAddress p0, NetworkInterface p1) throws IOException;
    
    @Override
    protected native void datagramSocketCreate() throws SocketException;
    
    @Override
    protected native void datagramSocketClose();
    
    protected native void socketNativeSetOption(int p0, Object p1) throws SocketException;
    
    @Override
    protected native Object socketGetOption(int p0) throws SocketException;
    
    @Override
    protected native void connect0(InetAddress p0, int p1) throws SocketException;
    
    protected native Object socketLocalAddress(int p0) throws SocketException;
    
    @Override
    protected native void disconnect0(int p0);
    
    @Override
    native int dataAvailable();
    
    private static native void init();
    
    static {
        init();
    }
}
